@@ -186,10 +186,24 @@ def add_uddf_input(parser):
     """
     Add list of UDDF files as input argument to a parser.
 
+    The function adds several options and arguments
+
+    - `-n` is added to fetch dives by their number
+    - multiple UDDF files can be specified
+    - each file can be preceded with `-k` option to indicate which dives
+      should be fetched from a file
+
+    The `-k` and `-n` options are glued with 'and' operator - use only one
+    of them if confused.
+
     :Parameters:
      parser
         ``argparse`` library parser.
     """
+    parser.add_argument('-n',
+            dest='dives',
+            help='fetch dives with their number (i.e. 40-42,45 are dives'
+                ' with dive number 40, 41, 42 and 45)')
     parser.add_argument('-k',
             dest='input',
             nargs=0,
@@ -199,7 +213,7 @@ def add_uddf_input(parser):
             nargs='+',
             action=UDDFInputAction,
             metavar='[-k dives] input',
-            help='dives from specified UDDF file (i.e.  1-3,6 is dive'
+            help='dives from specified UDDF file (i.e. 1-3,6 is dive'
                 ' 1, 2, 3, and 6 from a file, all by default)')
     parser.add_argument('input',
             nargs=argparse.REMAINDER,
