@@ -27,7 +27,9 @@ import lxml.etree as et
 import os.path
 import logging
 import itertools
+ichain = itertools.chain.from_iterable
 from itertools import zip_longest as lzip
+from operator import itemgetter
 import pkg_resources
 
 import kenozooid.uddf as ku
@@ -37,10 +39,10 @@ from kenozooid.units import K2C
 log = logging.getLogger('kenozooid.logbook')
 
 
-def find_dive_nodes(files, nodes=None):
+def find_dive_nodes(files, nodes=None, dives=None):
     """
-    Find dive nodes in UDDF files using optional numeric ranges as search
-    parameter.
+    Find dive nodes in UDDF files using optional numeric ranges or total
+    dive number as search parameters.
 
     The collection of dive nodes is returned.
 
@@ -49,14 +51,16 @@ def find_dive_nodes(files, nodes=None):
         Collection of UDDF files.
      nodes
         Numeric ranges of nodes, `None` if all nodes.
+     dives
+        Numeric range of total dive number, `None` if any dive.
 
     .. seealso:: :py:func:`parse_range`
     .. seealso:: :py:func:`find_dives`
     """
     nodes = [] if nodes is None else nodes
-    data = (ku.find(f, ku.XP_FIND_DIVES, nodes=q) \
+    data = (ku.find(f, ku.XP_FIND_DIVES, nodes=q, dives=dives) \
         for q, f in lzip(nodes, files))
-    return itertools.chain(*data)
+    return ichain(data)
 
 
 def find_dive_gas_nodes(files, nodes=None):
@@ -77,14 +81,14 @@ def find_dive_gas_nodes(files, nodes=None):
     nodes = [] if nodes is None else nodes
     data = (ku.find(f, ku.XP_FIND_DIVE_GASES, nodes=q) \
         for q, f in lzip(nodes, files))
-    nodes_by_id = ((n.get('id'), n) for n in itertools.chain(*data))
+    nodes_by_id = ((n.get('id'), n) for n in ichain(data))
     return dict(nodes_by_id).values()
 
 
-def find_dives(files, nodes=None):
+def find_dives(files, nodes=None, dives=None):
     """
-    Find dive data in UDDF files using optional node ranges as search
-    parameter.
+    Find dive data in UDDF files using optional node ranges or total dive
+    number as search parameters.
 
     The collection of dive data is returned.
 
@@ -93,11 +97,13 @@ def find_dives(files, nodes=None):
         Collection of UDDF files.
      nodes
         Numeric ranges of nodes, `None` if all nodes.
+     dives
+        Numeric range of total dive number, `None` if any dive.
 
     .. seealso:: :py:func:`parse_range`
     .. seealso:: :py:func:`find_dive_nodes`
     """
-    return (ku.dive_data(n) for n in find_dive_nodes(files, nodes))
+    return (ku.dive_data(n) for n in find_dive_nodes(files, nodes, dives))
         
 
 def list_dives(dives):
@@ -108,6 +114,7 @@ def list_dives(dives):
     each dive a tuple of formatted dive information
     is returned
 
+    - dive number, i.e. 102
     - date and time of dive, i.e. 2011-03-19 14:56
     - maximum depth, i.e. 6.0m
     - dive average depth, i.e. 2.0m
@@ -128,8 +135,8 @@ def list_dives(dives):
             avg_depth = ''
             if dive.avg_depth is not None:
                 avg_depth = '{:.1f}m'.format(dive.avg_depth)
-            yield (format(dive.datetime, FMT_DIVETIME), depth, avg_depth,
-                    duration, temp)
+            yield (dive.number, format(dive.datetime, FMT_DIVETIME), depth,
+                avg_depth, duration, temp)
         except TypeError as ex:
             log.debug(ex)
             log.warn('invalid dive data, skipping dive')
@@ -223,7 +230,7 @@ def upgrade_file(fin):
     return doc
 
 
-def copy_dives(files, nodes, lfile):
+def copy_dives(files, nodes, n_dives, lfile):
     """
     Copy dive nodes to logbook file.
 
@@ -234,6 +241,8 @@ def copy_dives(files, nodes, lfile):
         Collection of files.
      nodes
         Collection of dive ranges.
+     n_dives
+        Numeric range of total dive number, `None` if any dive.
      lfile
         Logbook file.
     """
@@ -242,7 +251,7 @@ def copy_dives(files, nodes, lfile):
     else:
         doc = ku.create()
 
-    dives = find_dive_nodes(files, nodes)
+    dives = find_dive_nodes(files, nodes, n_dives)
     gases = find_dive_gas_nodes(files, nodes)
 
     _, rg = ku.create_node('uddf:profiledata/uddf:repetitiongroup',
@@ -271,4 +280,52 @@ def copy_dives(files, nodes, lfile):
             log.debug('no dives copied')
 
 
+def enum_dives(files, total=1):
+    """
+    Enumerate dives with day dive number (when UDDF 3.2 is introduced) and
+    total dive number.
+
+    :Parameters:
+     files
+        Collection of UDDF files having dives to enumerate.
+     total
+        Start of total dive number.
+    """
+    fields = ('id', 'date')
+    queries = (
+        ku.XPath('@id'),
+        ku.XPath('uddf:informationbeforedive/uddf:datetime/text()'),
+    )
+    parsers = (str, lambda dt: ku.dparse(dt).date())
+
+    fnodes = ((f, n) for f in files for n in
+        ku.find(f, ku.XP_FIND_DIVES, nodes=None, dives=None))
+    data = ((f, ku.dive_data(n, fields, queries, parsers)) for f, n in fnodes)
+    data = ((item[0], item[1].id, item[1].date) for item in data) # flatten data
+    data = sorted(data, key=itemgetter(2))
+
+    # enumerate dives with _day_ dive number and flatten the groups
+    data = ichain(enumerate(g, 1) for k, g in
+        itertools.groupby(data, itemgetter(2)))
+
+    # enumerate dives with total dive number and transform into
+    #   { (f, id) => (n, k) }
+    cache = dict(((v[0], v[1]), (n, k)) for n, (k, v) in enumerate(data, total))
+
+    # update data
+    for f in files:
+        doc = ku.parse(f)
+        for n in ku.XP_FIND_DIVES(doc, nodes=None, dives=None):
+            id = n.get('id')
+            dnn = ku.xp_first(n, 'uddf:informationbeforedive/uddf:divenumber')
+            if dnn is None:
+                pn = ku.xp_first(n, 'uddf:informationbeforedive/uddf:internaldivenumber')
+                if pn is None:
+                    pn = ku.xp_first(n, 'uddf:informationbeforedive/uddf:datetime')
+                *_, dnn = ku.create_node('uddf:divenumber')
+                pn.addprevious(dnn)
+            dnn.text = str(cache[f, id][0])
+        ku.save(doc.getroot(), f)
+
+
 # vim: sw=4:et:ai
