@@ -96,6 +96,7 @@ XPath.__doc__ = """
 
 # XPath queries for default dive data
 XP_DEFAULT_DIVE_DATA = (
+    XPath('uddf:informationbeforedive/uddf:divenumber/text()'),
     XPath('uddf:informationbeforedive/uddf:datetime/text()'),
     XPath('uddf:informationafterdive/uddf:greatestdepth/text()'),
     XPath('uddf:informationafterdive/uddf:diveduration/text()'),
@@ -126,7 +127,7 @@ XP_DEFAULT_GAS_DATA =  (
 )
 
 # XPath query to locate dive profile sample
-XP_WAYPOINT = XPath('.//uddf:waypoint')
+XP_WAYPOINT = XPath('./uddf:samples/uddf:waypoint')
 # XPath query to locate gas mix
 XP_MIX = XPath('/uddf:uddf/uddf:gasdefinitions/uddf:mix')
 
@@ -178,7 +179,8 @@ XP_FIND_SITE = XPath('/uddf:uddf/uddf:divesite/uddf:site[' \
 
 # XPath query to find dives
 XP_FIND_DIVES = XPath('/uddf:uddf/uddf:profiledata' \
-    '/uddf:repetitiongroup/uddf:dive[in-range(position(), $nodes)]')
+    '/uddf:repetitiongroup/uddf:dive[in-range(position(), $nodes)' \
+    ' and in-range(uddf:informationbeforedive/uddf:divenumber/text(), $dives)]')
 
 # XPath query to find dive gases
 XP_FIND_DIVE_GASES = XPath('/uddf:uddf/uddf:gasdefinitions' \
@@ -384,10 +386,10 @@ def dive_data(node, fields=None, queries=None, parsers=None):
     .. seealso:: :py:func:`find_data`
     """
     if fields is None:
-        fields = ('datetime', 'depth', 'duration', 'temp', 'avg_depth',
-                'mode', 'profile')
+        fields = ('number', 'datetime', 'depth', 'duration', 'temp',
+            'avg_depth', 'mode', 'profile')
         queries = XP_DEFAULT_DIVE_DATA
-        parsers = (dparse, float, float, float, float, str, dive_profile)
+        parsers = (int, dparse, float, float, float, float, str, dive_profile)
 
     return find_data('Dive', node, fields, queries, parsers)
 
@@ -609,11 +611,20 @@ def in_range(ctx, pos, nodes):
     """
     if not nodes:
         return True
-    if 'in-range' not in ctx.eval_context:
+
+    if isinstance(pos, list):
+        if len(pos) == 0:
+            return False
+        if len(pos) != 1:
+            raise ValueError('Too many parameters')
+        pos = int(pos[0])
+
+    kf = 'in-range({})'.format(nodes)
+    if kf not in ctx.eval_context:
         nr = parse_range(nodes)
-        fstr = 'ctx.eval_context["in-range"] = lambda n: {}'.format(nr)
+        fstr = 'ctx.eval_context["{}"] = lambda n: {}'.format(kf, nr)
         exec(fstr)
-    return ctx.eval_context['in-range'](pos)
+    return ctx.eval_context[kf](pos)
 
 # register in-range XPath function
 ns = et.FunctionNamespace(None)
@@ -790,6 +801,8 @@ def set_data(node, queries, formatters=None, **data):
 
 def create_node(path, parent=None, append=True):
     """
+    TODO: get rid of parent, does not make sense
+
     Create a hierarchy of nodes using XML nodes path specification.
 
     Path is a string of node names separated by slash character, i.e. a/b/c
@@ -819,13 +832,16 @@ def create_node(path, parent=None, append=True):
     n = parent
     for t in tags:
         is_last = tags[-1] == t
+        k_exists = False
 
         k = None
         if n is not None:
             k = xp_first(n, t)
         if is_last or k is None:
             k = et.Element(T(t))
-        if n is not None:
+        elif k is not None:
+            k_exists = True
+        if n is not None and not k_exists:
             if append:
                 n.append(k)
             else:
@@ -1230,8 +1246,10 @@ def reorder(doc):
         raise ValueError('No profile data to reorder')
     pd = profiles[0]
 
-    nodes = find('//uddf:dive')
-    times = find('//uddf:dive/uddf:informationbeforedive/uddf:datetime/text()')
+    q = '/uddf:uddf/uddf:profiledata/uddf:repetitiongroup/uddf:dive'
+    qt = q + '/uddf:informationbeforedive/uddf:datetime/text()'
+    nodes = find(q)
+    times = find(qt)
 
     dives = {}
     for n, t in zip(nodes, times):
